<?php
if(!class_exists("Gc_Message_Box_Controller")){
    class Gc_Message_Box_Controller implements Gc_MessageBox_Controller_Interface {
        protected $namespace;
        protected $first_appear;
        protected $enable_repeat;
        protected $repeat_after;
        protected $gci = 0;
        protected $inloop = false;
        protected $renderer = null;
		protected $volt = false;
        
        public function __construct($namespace){
            $this->namespace = $namespace;

        }

        public function initialize(){
            $options = GCCF::create("Option_Repository_Factory")->get_instance()->get_namespace($this->namespace);
            $this->first_appear = $options->get("first_appear");
            $this->enable_repeat = $options->get("enable_repeat");
            $this->repeat_after = $options->get("repeat_after");
            $this->renderer = new Gc_Message_Box_Renderer($this->namespace, $this);

        }

        public function initialize_hooks(){
            add_action( 'init', array($this, 'scripts_init') );
            add_action( 'the_post', array($this,'put_after_the_post'), 1 );
            add_action( 'loop_start', array($this,'put_loop_start'), 1 );
            add_action( 'loop_end', array($this,'put_loop_end'), 1 );
			add_action( 'pre_get_posts', array($this,'reset_loop'), 1 );
            add_filter( 'comments_array', array($this,'comments_list_before'), 1 );
        }

        public function handle_request(){

        }
        
        public function scripts_init(){
            wp_register_style( 'gc_message_box_generated_style', plugins_url('gc-message-box/style-gc-message-box.php') );
            wp_enqueue_style( 'gc_message_box_generated_style' );

            wp_enqueue_script( 'jquery' );
            
            wp_register_style( 'google_webfonts', 'http://fonts.googleapis.com/css?family=Droid+Sans:400,700|Lato:100,300,400,700,900,100italic,300italic,400italic,700italic,900italic|PT+Sans:400,700,400italic,700italic|Bitter:400,700,400italic|Droid+Serif:400,700,700italic,400italic|Open+Sans:300italic,400italic,600italic,700italic,800italic,400,800,700,600,300|Oswald:400,700,300|Open+Sans+Condensed:300,300italic,700|Yanone+Kaffeesatz:400,700,300,200|Roboto:400,900italic,700italic,900,700,500italic,500,400italic,300italic,300,100italic,100&subset=latin,latin-ext,cyrillic,cyrillic-ext,greek-ext,greek,vietnamese' );
            wp_enqueue_style('google_webfonts');            

        }
		
		public function reset_loop() {
			wp_reset_query();
		}

        public function put_loop_start($query) {
			global $wp_the_query;
			
            $this->gci = 0;
            if ( ($query === $wp_the_query) && $query->is_main_query()) {
                $this->inloop = true;
            }
        }


        public function put_after_the_post(){
            //Loop end
            if(!$this->inloop){
                return;
            }
            //Post page
            if(is_single()){
                return;
            }
            //Appear here test
            if (!$this->is_appear_here_enabled()) {
                return;
            }

            if($this->is_loop_counter_equal_first_appear()) {
                $this->renderer->render($this->gci);
            }
            if($this->is_repeat_enable()){
                if($this->is_loop_counter_less_first_appear()){
                    if($this->is_repeat_position()){
                        echo $this->renderer->render($this->gci);
                    }
                }
            }

            $this->gci++;

        }

        public function put_loop_end(  ) {
            $this->inloop = false;
        }

        public function comments_list_before($my_comments){
            
            if ($this->is_appear_here_enabled()) {
                echo $this->renderer->render($this->gci);
            }
            return $my_comments;
        }
        
        public function is_list() {
            return (is_home() || is_category() || is_search() || is_author() || is_archive() || is_tag());
        }
        
        public function is_article() {
            return is_single();
        }

        public function is_page() {
            return is_page();
        }


        protected function is_loop_counter_equal_first_appear(){
            return ($this->gci == $this->first_appear->get_value());
        }

        protected function is_loop_counter_less_first_appear(){
            return ((int)$this->first_appear->get_value() < $this->gci);
        }

        protected function is_repeat_enable(){
            return ($this->enable_repeat->is_checked());
        }

        protected function is_repeat_position(){
            return ((($this->gci - $this->first_appear->get_value()) % $this->repeat_after->get_value()) == 0);
        }

        protected function is_appear_here_enabled() {
            $options = GCCF::create("Option_Repository_Factory")->get_instance()->get_namespace($this->namespace);
            if ($options->get("appear_here")->is_checked()) {
                return true;
            }
            $page_functions = array(
                "is_home"       =>  "appear_here_home",
                "is_category"   =>  "appear_here_category",
                "is_tag"        =>  "appear_here_tag",
                "is_archive"    =>  "appear_here_archive",
                "is_author"     =>  "appear_here_author",
                "is_search"     =>  "appear_here_search",
                "is_single"     =>  "appear_here_article",
                "is_page"       =>  "appear_here_pages"
            );
            $appear_pages = $options->filter("is_appear", true);
            foreach ($page_functions as $function => $opt) {
                if($function()) {
                    return ($appear_pages[$opt]->is_checked());
                }
            }   
            return false;
        }
    }
}